@@ -72,6 +72,7 @@ class wxListBox;
 class wxGrid;
 class ShuttlePrefs;
 class ReadOnlyText;
+class SpinControl;
 
 class WrappedType;
 
@@ -260,6 +261,9 @@ public:
    wxSlider * AddVSlider(const TranslatableString &Prompt, int pos, int Max);
    wxSpinCtrl * AddSpinCtrl(const TranslatableString &Prompt,
       int Value, int Max, int Min);
+   SpinControl* AddSpinControl(
+      const wxSize& size, const TranslatableString& Prompt, double Value,
+      double Max, double Min);
    wxTreeCtrl * AddTree();
 
    // Pass the same initValue to the sequence of calls to AddRadioButton and
@@ -435,6 +439,9 @@ public:
 
    wxSpinCtrl * TieSpinCtrl( const TranslatableString &Prompt,
       int &Value, const int max, const int min = 0 );
+   SpinControl* TieSpinControl(
+      const wxSize& size, const TranslatableString& Prompt, double& Value,
+      const double max, const double min = 0);
 
 
 //-- Variants of the standard Tie functions which do two step exchange in one go
@@ -569,6 +576,9 @@ private:
       WrappedType & WrappedRef, const int max, const int min = 0 );
    wxSpinCtrl * DoTieSpinCtrl( const TranslatableString &Prompt,
       WrappedType & WrappedRef, const int max, const int min = 0 );
+   SpinControl* DoTieSpinControl(
+      const wxSize& size, const TranslatableString& Prompt,
+      WrappedType& WrappedRef, const double max, const double min = 0);
 
    std::vector<EnumValueSymbol> mRadioSymbols;
    wxString mRadioSettingName; /// The setting controlled by a group.
